"""
creates a skymap in healpix representation with data from GAIA.


    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement..


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""

import os.path
import sys
from operator import itemgetter
import math
import healpy as hp
from nsb.gaia import database_query


def loadCat(level):
    cat = loadFile(os.path.join(os.path.dirname(os.path.realpath(__file__)), "ducaticat.txt"), level)
    home = os.path.join(os.path.expanduser("~"), ".nsb/")
    catfile = os.path.join(home, "gaiacat.txt")
    if os.path.isfile(catfile):
        cat += loadFile(catfile, level)
    else:
        print('GAIA: No %s found on disc' % catfile)
        i = eval(input('GAIA: Will download the gaia catalog\n'
                       'GAIA: up to which magnitude shall we go?\n'
                       'GAIA: recommended ~15.0 (about 4GB of data) for testing approx 8.0 (about 6MB) will do it...\n'))
        cat += getCat(3.0, i, level) # 14.5
    print('GAIA: now these %i lines have to be sorted! ...' % len(cat))
    cat.sort(key=itemgetter(0))
    return cat


def getCat(magmin, magmax, level):
    try:
        query = database_query.query(magmin, magmax)
        query.submit()
        query.checkStatus()
        data = query.getResult()
        print('GAIA: now processing...')
        catlist = []
        home = os.path.join(os.path.expanduser("~"), ".nsb/")
        catfile = os.path.join(home, "gaiacat.txt")
        with open(catfile, 'a') as f:
            for line in data:
                f.write(line + '\n')
                splitted = line.split(',')
                catlist.append([int(splitted[0]), float(splitted[1]), float(splitted[2]), float(splitted[3]), float(splitted[4])])
                catlist[-1][0] = int(catlist[-1][0] / (2**35 * 4**(12 - level)))
            if len(data) == 3000000:
                print('GAIA: They didnt send us everything. Lets go again.')
                return catlist + getCat(catlist[-1][4], magmax, level)
            else:
                print('GAIA: We got everything.')
                return catlist
    except IOError as e:
        raise e


def loadFile(catalog, level):
    """
    load a txt file with catalog values.
    format:
    source_id,ra,dec,phot_g_mean_flux,phot_g_mean_mag
    """
    try:
        print('GAIA: loading catalog %s from disc...' % catalog)
        catlist = []
        with open(catalog, 'r') as catfile:
            for line in catfile.readlines():
                splitted = line.split(',')
                try:
                    catlist.append([int(splitted[0]), float(splitted[1]), float(splitted[2]), float(splitted[3]), float(splitted[4])])
                    catlist[-1][0] = int(catlist[-1][0] / (2**35 * 4**(12 - level)))
                except Exception as e:
                    print('GAIA: ERROR in ' + str(catalog) + ': '+ str(e))
                    print('GAIA: The problematic line was:\n\t' + str(line))
        return catlist

    except IOError as e:
        raise e


"""
-   HEALpix level 12 = source_id / 34359738368

-   HEALpix level 11 = source_id / 137438953472

-   HEALpix level 10 = source_id / 549755813888

-   HEALpix level n = source_id / 2 ^ 35 * 4 ^ (12 - level)
"""


def binarySearch(value, lis):
    left = 0
    right = len(lis) - 2
    while True:
        if (left > right):
            return left
        test = int((left + right) / 2.0)
        testvalue = lis[test][0]
        if testvalue > value:
            right = test - 1
        elif testvalue < value:
            left = test + 1
        elif testvalue == value:
            return test


def getAllwithIndex(listindex, cat):
    lower = listindex
    upper = listindex
    while cat[lower][0] == cat[listindex][0]:
        lower -= 1
        if lower < 0:
            break
    while cat[upper][0] == cat[listindex][0]:
        upper += 1
        if upper > len(cat) - 1:
            break
    return cat[lower + 1: upper]


def getTotalFlux(healpixid, cat):
    """
    Get the Flux in that specific healpixel
    """
    seed = binarySearch(healpixid, cat)
    stars = getAllwithIndex(seed, cat)

    flux = 0.0
    for source in stars:
        flux += source[3]
    return flux, len(stars)


def getSurfaceBrightness(flux, area):
    if flux > 0:
        m = -2.5 * math.log10(flux) + 25.5248
        return m + 2.5 * math.log10(area)
    else:
        return 100.0


def createMap(level):
    cat = loadCat(level)
    catalogname = 'healpixmap_level' + str(level) + '.txt'
    home = os.path.join(os.path.expanduser("~"), ".nsb/")
    filepath = os.path.join(home, catalogname)
    print('GAIA: now creating ' + filepath)
    with open(filepath, 'w') as f:
        pixel = hp.nside2npix(2**level)
        for healpixid in range(pixel):
            ra, dec = hp.pix2ang(2**level, healpixid, nest=True, lonlat=True)
            deg2 = hp.nside2pixarea(2**level, degrees=True)
            arcsec2 = deg2 * 60**2 * 60**2
            flux, number = getTotalFlux(healpixid, cat)
            brightness = getSurfaceBrightness(flux, arcsec2)
            f.write(str(healpixid) + ', ' +
                    str(ra) + ', ' +
                    str(dec) + ', ' +
                    str(flux) + ', ' +
                    str(brightness) + ', ' +
                    str(arcsec2) + ', ' +
                    str(number) + '\n')
    print("GAIA: Succesfully created " + catalogname)


# Here comes the main
if __name__ == '__main__':
    level = int(sys.argv[1])
    createMap(level)
    print('GAIA: done.')




#!/usr/bin/env python3
"""
allskymaps python package.

Draws Allsky Nightsky Images with stars from the GAIA catalog

    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement.

and draws moonlight corresponding to the Model from Krisciunas et al.

   author = {{Krisciunas}, K. and {Schaefer}, B.~E.},
    title = "{A model of the brightness of moonlight}",
  journal = {\pasp},
     year = 1991,
    month = sep,
   volume = 103,
    pages = {1033-1039},
      doi = {10.1086/132921},
   adsurl = {http://adsabs.harvard.edu/abs/1991PASP..103.1033K},


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""


import sys
import inspect
import ephem
import os.path
import math

from .help import PlatformHelp
from .input_options import ReadOptions
from .fitsimage import FitsImage
from .config import TheConfiguration
from .nsbtools import makeDateString, plotMaps
from .mypycat import mypycat
from .gaia import Gaia


def main():
    home = os.path.join(os.path.expanduser("~"), ".nsb/")
    if not os.path.exists(home):
        os.makedirs(home)

    Help = PlatformHelp()
    Options = ReadOptions(sys.argv)
    try:
        assert(Options.show_help == False)
    except:
        print(inspect.stack()[0][2:4][::-1])
        # Show help and halt
        Help.show_help()
        raise SystemExit

    Help.show_title()
    Configuration = TheConfiguration()
    if Options.altconfset:
        Configuration.readConfig(Options.conf)
    else:
        Configuration.readStandardConfig()

    # Lets have a look what we gonna do today...

    # try to get all the values from user input (Options)
    # if it fails, because there was no input -> use the values from Configfile.

    try:
        size = Options.imageSize
        print('* user input Imagesize of %i pixels' % size)
    except Exception as e:
        size = int(Configuration.config['image_size'])
        print('* Imagesize of %i pixels from config file' % size)
    try:
        if Options.source_set:
            mpc = mypycat()
            source = mpc.get(Options.tracked_source_name)
            alt, az = 0, 0 # has to be calculated later
            print('* user input pointing on %s' % source.name)
        else:
            alt = Options.alt
            az = Options.az
            source = None
            print('* user input Alt/Az of %.1f|%.1f' % (alt, az))
    except Exception as e:
        alt = float(Configuration.config['alt'])
        az = float(Configuration.config['az'])
        source = None
        print('* Alt/Az of %.1f|%.1f from config file' % (alt, az))
    try:
        fov = Options.fov
        print('* user input Image fov of %.1f' % fov)
    except Exception as e:
        fov = float(Configuration.config['fov'])
        print('* Image fov of %.1f from config file' % fov)
    try:
        level = Options.healpixlevel
        print('* user input HealPixLevel of %i' % level)
    except Exception as e:
        level = int(Configuration.config['healpixlevel'])
        print('* HealPixLevel of %i from config file' % level)
    try:
        lat, lon = float(Options.lat), float(Options.lon)
        print('* user input location: Lat %s Lon %s' % (lat, lon))
    except Exception as e:
        lat, lon = float(Configuration.config['Lat']), float(Configuration.config['Lon'])
        print('* location from config file: Lat %s Lon %s' % (lat, lon))
    try:
        elevation = float(Options.elevation)
        print('* user input elevation: %.1f m ' % elevation)
    except Exception as e:
        elevation = float(Configuration.config['elevation'])
        print('* elevation from config file: %.1f m' % elevation)
    try:
        B_zero = Options.B_zero
        print('* user input B_zero = %.3f' % B_zero)
    except Exception as e:
        B_zero = float(Configuration.config['B'])
        print('* B_zero = %.3f from config file ' % B_zero)
    try:
        k = Options.extinction
        print('* user input k=%.3f' % k)
    except Exception as e:
        k = float(Configuration.config['k'])
        print('* Extinction from config file k=%.3f' % k)
    try:
        gauss = Options.gauss
        if gauss > 0:
            print('* user input gaussian smoothing with %.1f' % gauss)
        else:
            print('* no smoothing')
    except Exception as e:
        gauss = float(Configuration.config['gauss'])
        if gauss > 0:
            print('* Gaussian smoothing with %.1f from config' % gauss)
        else:
            print('* no smoothing')
    try:
        use_mag = Options.use_mag
        print('* user input units mag/arcsec^2')
    except Exception as e:
        um = Configuration.config['use_mag']
        if um in ('True', 'true', '1', 'yes', 'Yes', 't', 'T'):
            use_mag = True
        else:
            use_mag = False
        print('* units specified in config file: use_mag = %s' % use_mag)
    try:
        output_dir = Options.output_dir
        print('* user specified output to: %s' % output_dir)
    except Exception as e:
        output_dir = ""
    try:
        date_and_time = ephem.Date(Options.time)
        print('* user input Date of %s' % date_and_time)
    except Exception as e:
        date_and_time = ephem.Date(Configuration.config['time'])
        print('* Date from config file %s' % date_and_time)

    """
    Let the Drawing begin!
    """

    Image_allsky = FitsImage(size, 180, 90.0, 0.0, None, lon, lat, elevation, date_and_time, B_zero, k, use_mag)
    Image_FOV = FitsImage(size, fov, alt, az, source, lon, lat, elevation, date_and_time, B_zero, k, use_mag)

    print("* Moon at \tzen %.2f\taz %.2f\twith a phase of %.2f %% (%.2f deg)" % (
          math.degrees(Image_FOV.zen_moon), math.degrees(Image_FOV.az_moon), 100 * Image_FOV.moon.moon_phase,
          math.degrees(Image_FOV.moon_alpha)))

    gaiamap = Gaia(level)
    Image_FOV.drawModel(gaiamap)
    Image_allsky.drawModel(gaiamap)

    median, mean = Image_allsky.getStatistics()
    print('Median All-Sky Brightness: \t %.2f %s\n'
          'Mean All-Sky Brightness: \t %.2f %s' % (median, Image_allsky.units, mean, Image_allsky.units))
    median, mean = Image_FOV.getStatistics()
    print('Median FOV Brightness: \t %.2f %s\n'
          'Mean FOV Brightness: \t %.2f %s' % (median, Image_FOV.units, mean, Image_FOV.units))

    if Options.savefits:
        Image_FOV.fitsfile.writeto(output_dir
                                   + "NSB_of_"
                                   + makeDateString(date_and_time)
                                   + "_FOV.fits", 'exception', True)
        Image_allsky.fitsfile.writeto(output_dir
                                      + "NSB_of_"
                                      + makeDateString(date_and_time)
                                      + "_All-Sky.fits", 'exception', True)


    if Options.noscreen:
        pass
    else:
        plotMaps([Image_allsky, Image_FOV])#, Image_ObsModel])

    print("done.\n")


if __name__ == '__main__':
    main()



"""
allskymaps python package.

Draws Allsky Nightsky Images with stars from the GAIA catalog

    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement.

and draws moonlight corresponding to the Model from Krisciunas et al.

   author = {{Krisciunas}, K. and {Schaefer}, B.~E.},
    title = "{A model of the brightness of moonlight}",
  journal = {\pasp},
     year = 1991,
    month = sep,
   volume = 103,
    pages = {1033-1039},
      doi = {10.1086/132921},
   adsurl = {http://adsabs.harvard.edu/abs/1991PASP..103.1033K},


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""
from .__about__ import __shortname__, __version__, __long_description__

class PlatformHelp():
    def __init__(self):
        self.make_title()
        self.make_welcome()
        self.make_requisites()
        self.make_options()

    def make_title(self):
        nameversion = 10 * '#' + 3 * ' ' + __shortname__ + ' v' + __version__ + 3 * ' ' + 10 * '#'
        self.separator = len(nameversion) * '#'
        self.title = nameversion

    def make_welcome(self):
        self.welcome = 'Welcome to ' + __shortname__ + ' (' + __long_description__ + ')\n'

    def make_requisites(self):
        self.requisites = \
            'Requisites: Python 3.5; Scipy; Numpy; Ephem; Astropy; tqdm\n\n'

#
    def make_options(self):
        self.options = \
            '--h --help prints this help message\n\n' + \
            'options (can all be set in config):\n' + \
            '--t --time DATETIME: \n' + \
            '   time and date for which the map should be drawn\n' + \
            '   format: 2010/12/24 23:59:59\n' + \
            '--o --output /OUTPUT/DIRECTORY/\n' + \
            '   for saving results in different place than working directory\n' + \
            '   filenames will be generated automatically\n' + \
            '--s --size IMAGESIZE:\n' + \
            '   set the output Imagesize in Pixels\n' + \
            '--fov FIELDOFVIEW:\n' + \
            '   set the (alt) fov\n' + \
            '--altaz ALT AZ:\n' + \
            '   set the observation position in the sky manually\n' + \
            '--q --source SOURCENAME:\n' + \
            '   set the observation position automatically on a source by name\n' + \
            '   i.e.: "crab", "moon"\n' + \
            '--l --location LOCATION:\n' + \
            '   The observers coordinates (Lon Lat) on earth and elevation\n' + \
            '   format: 16.5028 -23.27280 1800.0\n' + \
            '--b --brightness NSB zenith brightness:\n' + \
            '   dark night sky zenith brightness in nanoLambert. Needed for model\n' + \
            '   format: 100.0 \n' + \
            '--k EXTINCTION:\n' + \
            '   Extinction in mags per air mass. Needed for model\n' + \
            '   format: 0.55\n' + \
            '--hp LEVEL:\n' + \
            '   HealPix Level for the gaia catalog to be plotted\n' + \
            '   Integer in range [1:12]\n' + \
            '--gauss KERNELSIZE:\n' + \
            '   Gaussian Kernel in pixel to smoothen the model images\n' + \
            '   format: 1.5\n' + \
            '\nFlags:\n' + \
            '-mags\n' + \
            '   use mags per arcsec^2 as output units (instead of nanoLamberts)\n' + \
            '-savefits\n' + \
            '   save Brightness Map(s) as *.fits file(s)\n' + \
            '-noscreen\n' + \
            '   do not show the Image using matplotlib plt\n\n'

#
    def show_help(self):
        print(
            self.separator + '\n' + self.title + '\n' + self.separator + '\n' + self.welcome +
            self.requisites + self.options + self.separator)

    def show_title(self):
        print(
            self.separator + '\n' + self.title + '\n' + self.separator)

    def incorrect_parameter(self, parameter):
        print('ERROR. Incorrect parameter: ' + str(parameter))
        self.show_help()

    def date_or_file_input_error(self):
        print('ERROR. File input')
        self.show_help()

    def no_parameters_error(self):
        print('ERROR. No input parameters specified')
        self.show_help()
###
###


"""
creates a skymap in healpix representation with data from GAIA.


    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement..


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""

import http.client
import urllib.parse
import time
from xml.dom.minidom import parseString


class query(object):
    """docstring for query"""
    def __init__(self, magmin, magmax):
        self.magmin = magmin
        self.magmax = magmax
        self.host = "gea.esac.esa.int"
        self.port = 80
        self.pathinfo = "/tap-server/tap/async"
        self.timestart = time.time()
        self.jobid = ''

    def submit(self):
        """
        Creates a ADQL query at gaia archive
        """

        params = urllib.parse.urlencode({
            "REQUEST": "doQuery",
            "LANG": "ADQL",
            "FORMAT": "csv",
            "PHASE": "RUN",
            "JOBNAME": "Any name (optional)",
            "JOBDESCRIPTION": "Any description (optional)",
            "QUERY": "SELECT source_id,ra,dec,phot_g_mean_flux,phot_g_mean_mag FROM gaiadr1.gaia_source WHERE phot_g_mean_mag >= " +
            str(self.magmin) + " and phot_g_mean_mag<= " + str(self.magmax)})

        headers = {
            "Content-type": "application/x-www-form-urlencoded",
            "Accept": "text/plain"}

        print('GAIA: We submitted a query to ESA asking for all stars with Magnitude in range [%.2f, %.2f]' % (self.magmin, self.magmax))

        connection = http.client.HTTPConnection(self.host, self.port)
        connection.request("POST", self.pathinfo, params, headers)

        # Status
        response = connection.getresponse()
        print("GAIA: Status: " + str(response.status), "Reason: " + str(response.reason))

        # Server job location (URL)
        location = response.getheader("location")
        print("GAIA: Location: " + location)

        # Jobid
        self.jobid = location[location.rfind('/') + 1:]
        print("GAIA: Job id: " + self.jobid)

        timestart = time.time()
        print("GAIA: Started at %s" % time.strftime("%H:%M:%S", time.localtime(timestart)))

        connection.close()

    def checkStatus(self):
        """
        Check job status, wait until finished
        """
        sleep = 0.25
        while True:
            connection = http.client.HTTPConnection(self.host, self.port)
            connection.request("GET", self.pathinfo + "/" + self.jobid)
            response = connection.getresponse()
            data = response.read()
            # XML response: parse it to obtain the current status
            dom = parseString(data)
            phaseElement = dom.getElementsByTagName('uws:phase')[0]
            phaseValueElement = phaseElement.firstChild
            phase = phaseValueElement.toxml()
            # Check finished
            if phase == 'COMPLETED':
                print("GAIA: Status: %s\t --> finished after %i seconds" % (phase, time.time() - self.timestart))
                break
            print("GAIA: Status: %s\t since %i seconds" % (phase, time.time() - self.timestart))
            # wait and repeat
            time.sleep(sleep)
            if sleep < 10:
                sleep *= 2
        connection.close()

    def getResult(self):
        print('GAIA: Now downloading the results...')
        time2 = time.time()
        connection = http.client.HTTPConnection(self.host, self.port)
        connection.request("GET", self.pathinfo + "/" + self.jobid + "/results/result")
        response = connection.getresponse()
        data = response.read().decode().split('\n')
        print('GAIA: Downloaded %i lines of data in %.1f seconds' % (len(data) - 2, time.time() - time2))
        connection.close()
        return data[1:-1]




"""
allskymaps python package.

Draws Allsky Nightsky Images with stars from the GAIA catalog

    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement.

and draws moonlight corresponding to the Model from Krisciunas et al.

   author = {{Krisciunas}, K. and {Schaefer}, B.~E.},
    title = "{A model of the brightness of moonlight}",
  journal = {\pasp},
     year = 1991,
    month = sep,
   volume = 103,
    pages = {1033-1039},
      doi = {10.1086/132921},
   adsurl = {http://adsabs.harvard.edu/abs/1991PASP..103.1033K},


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""


import os
from .nsbtools import makeDate



class ReadOptions():
    def __init__(self, input_options):
        # Lambda: http://docs.python.org/release/2.5.2/tut/node6.html [4.7.5]
        self.options = {'--h': lambda: 'show_help', '--help': lambda: 'show_help',
                        '--o': lambda: 'output', '--output': lambda: 'output',
                        '--t': lambda: 'time', '--time': lambda: 'time',
                        '--l': lambda: 'location', '--location': lambda: 'location',
                        '--altaz': lambda: 'altaz',
                        '--q': lambda: 'source', '--source': lambda: 'source',
                        '--fov': lambda: 'fov',
                        '--s': lambda: 'size', '--size': lambda: 'size',
                        '--b': lambda: 'brightness', '--brightness': lambda: 'brightness',
                        '--hp': lambda: 'healpixlevel',
                        '--k': lambda: 'extinction',
                        '--use': lambda: 'altconf',
                        '-mags': lambda: 'mag',
                        '-noscreen': lambda: 'noscreen',
                        '-savefits': lambda: 'savefits'
                        }

        self.show_help = False
        self.input_options = input_options
        self.inputfile_set = False
        self.savestats = False
        self.savefits = False
        self.noscreen = False
        self.altconfset = False
        self.source_set = False

        while len(self.input_options) > 1:
            input_option = self.options.get(self.input_options[1], lambda: None)()
            if input_option == 'show_help':
                self.show_help = True
                # Stop reading options. Program will halt
                self.input_options = []

            elif input_option == 'time':
                try:
                    self.time = makeDate(input_options[2], input_options[3])

                    #self.time = ephem.Date((year, month, day, hour, minute, second))
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'location':
                self.lat, self.lon, self.elevation = input_options[2], input_options[3], input_options[4]
                self.input_options.remove(self.input_options[1])
                self.input_options.remove(self.input_options[1])
                self.input_options.remove(self.input_options[1])
                self.input_options.remove(self.input_options[1])

            elif input_option == 'size':
                try:
                    self.imageSize = int(input_options[2])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'fov':
                try:
                    self.fov = float(input_options[2])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'source':
                try:
                    self.source_set = True
                    self.tracked_source_name = input_options[2]
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'altaz':
                try:
                    self.alt = float(input_options[2])
                    self.az = float(input_options[3])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'healpixlevel':
                try:
                    self.healpixlevel = int(input_options[2])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'extinction':
                try:
                    self.extinction = float(input_options[2])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'brightness':
                try:
                    self.B_zero = float(input_options[2])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'gauss':
                try:
                    self.gauss = float(input_options[2])
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                except Exception as e:
                    self.incorrect_parameter()
                    print(e)

            elif input_option == 'mag':
                self.use_mag = True
                self.input_options.remove(self.input_options[1])

            elif input_option == 'savefits':
                self.savefits = True
                self.input_options.remove(self.input_options[1])

            elif input_option == 'noscreen':
                self.noscreen = True
                self.input_options.remove(self.input_options[1])

            elif input_option == 'output':
                if os.path.exists(input_options[2]):
                    self.output_dir = input_options[2]
                    self.input_options.remove(self.input_options[1])
                    self.input_options.remove(self.input_options[1])
                else:
                    print('Output path doesnt exist!')
                    self.incorrect_parameter()

            elif input_option == 'altconf':
                self.conf = input_options[2]
                self.altconfset = True
                self.input_options.remove(self.input_options[1])
                self.input_options.remove(self.input_options[1])

            else:
                self.incorrect_parameter()


    def incorrect_parameter(self):
        print('\nERR: Incorrect parameter ' + str(self.input_options[1]))
        self.input_options = []
        self.show_help = True

    def no_parameters(self):
        print('\nERR: Need at least a date and time')
        self.input_options = []
        self.show_help = True
#



"""
allskymaps python package.

Draws Allsky Nightsky Images with stars from the GAIA catalog

    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement.

and draws moonlight corresponding to the Model from Krisciunas et al.

   author = {{Krisciunas}, K. and {Schaefer}, B.~E.},
    title = "{A model of the brightness of moonlight}",
  journal = {\pasp},
     year = 1991,
    month = sep,
   volume = 103,
    pages = {1033-1039},
      doi = {10.1086/132921},
   adsurl = {http://adsabs.harvard.edu/abs/1991PASP..103.1033K},


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""


import numpy as np
import time
import math
import matplotlib.pyplot as plt
import matplotlib.cm as cm


def writeToDataFile(filename, values):
    '''
    Very simple Function to write (append) some values to a file
    '''
    f = open(filename, 'a')
    f.write(values)
    f.close()


def plotMaps(images):
    print('Please close Plot Window to proceed (terminate)...')
    """
    vmin, vmax = np.nanmin(images[0].fitsfile.data), np.minimum(2 * np.nanmedian(images[0].fitsfile.data),
                                                                np.nanmax(images[0].fitsfile.data))
    plt.imshow(images[0].fitsfile.data, cmap=cm.viridis, vmin=vmin, vmax=vmax)
    plt.colorbar()
    plt.gca().get_xaxis().set_visible(False)
    plt.gca().axes.get_yaxis().set_visible(False)
    plt.title('Allskymaps for %s\n(Lon %.4f | Lat %.4f)' % (images[0].site.date, math.degrees(images[0].site.lon), math.degrees(images[0].site.lat)))
    plt.show()
    """

    fig, (sub1, sub2) = plt.subplots(1, 2)
    fig.suptitle('skymaps for %s\n(Lon %.4f | Lat %.4f)' % (
    images[0].site.date, math.degrees(images[0].site.lon), math.degrees(images[0].site.lat)))

    sub1.get_xaxis().set_visible(False)
    sub1.axes.get_yaxis().set_visible(False)
    sub2.get_xaxis().set_visible(False)
    sub2.axes.get_yaxis().set_visible(False)

    sub1.set_title('All-Sky')
    sub2.set_title('FOV %.1f°\nalt %.1f az %.1f' % (images[1].fov,math.degrees(images[1].alt), math.degrees(images[1].az)))

    vmin, vmax = np.nanmin(images[0].fitsfile.data), np.minimum(2 * np.nanmedian(images[0].fitsfile.data),
                                                                np.nanmax(images[0].fitsfile.data))
    im1 = sub1.imshow(images[0].fitsfile.data, cmap=cm.viridis, vmin=vmin, vmax=vmax)
    vmin, vmax = np.nanmin(images[1].fitsfile.data), np.minimum(2 * np.nanmedian(images[1].fitsfile.data),
                                                                np.nanmax(images[1].fitsfile.data))
    im2 = sub2.imshow(images[1].fitsfile.data, cmap=cm.viridis, vmin=vmin, vmax=vmax)

    fig.colorbar(im1, ax=sub1, orientation="horizontal")
    fig.colorbar(im2, ax=sub2, orientation="horizontal")

    plt.show()


def makeDateString(date):

    d = date.tuple()
    date_string = ''
    for x in d:
        if float(x) < 1:
            x = 0
        if len(str(int(x))) > 1:
            date_string += str(int(x))
        elif len(str(int(x))) == 1:
            date_string += '0' + str((x))
    return date_string

def makeDate(datestring, timestring):
    if datestring in ['today', 'heute']:
        year = time.strftime("%Y")
        month = time.strftime("%m")
        day = time.strftime("%d")
    else:
        year = datestring[0:4]
        month = datestring[5:7]
        day = datestring[8:]

    if timestring in ['now', 'jetzt']:
        hour = time.strftime("%H")
        minute = time.strftime("%M")
        second = time.strftime("%S")
    else:
        hour = timestring[0:2]
        minute = timestring[3:5]
        second = timestring[6:]

    return "" + str(year) + "/" + str(month) + "/" + str(day) + " " \
                + str(hour) + ":" + str(minute) + ":" + str(second)


def nLbMag(B):
    '''
    nanoLambert to Magnitude conversion
    '''
    return -2.5 * math.log10(1.0 / np.pi * 0.00001 * B / 108400.0)


def MagnLb(M):
    '''
    Magnitude to nanoLambert conversion
    '''
    return np.pi * 100000 * 108400 * 10**(-0.4 * M)


def greatCircle(Zen, phi, Zen2, phi2):
    '''
    Calculates the great Circle Distance of two pairs of spheric coordinates
    Works for GPS Coordinates as well as for Zen / Az (not ALT !)
    '''
    return math.acos(math.sin(np.pi / 2. - Zen) * math.sin(np.pi / 2. - Zen2) +
                     math.cos(np.pi / 2. - Zen) * math.cos(np.pi / 2. - Zen2) * math.cos(phi - phi2))


def X(Z):
    return 0.4 + 0.6 * (1 - 0.96 * (math.sin(Z))**2)**(-0.5)


def f(rho):
    r = math.degrees(rho)
    A = 5.2313
    B = 0.9351
    F = 5.9014
    return 10**A * (B + (math.cos(rho))**2) + 10**(F - r / 40)


def I_moon(moon_alpha):
    a = math.degrees(moon_alpha)
    return 10**(-0.4 * (3.84 + 0.026 * math.fabs(a) + 4 * 10**(-9) * a**4))


def B_sky(Zen, phi, B_zero, k):
    """
    Krisciunas et. al Sky-only-Model
    Brightness of the Sky at certain position without moon
    """
    if Zen < np.pi / 2:
        return B_zero * X(Zen) * 10**(-0.4 * k * (X(Zen) - 1))
    else:
        return 0


def B_moon(Zen, phi, Zen_moon, phi_moon, k, moon_alpha):
    """
    Krisciunas et. al Moon-Model
    only valid for moon above horizon. it gives values for negative moon alt,
    but they have no physical meaning.
    --> so in that case better return 0
    """
    if Zen_moon < np.pi / 2:
        rho = greatCircle(Zen, phi, Zen_moon, phi_moon)
        return f(rho) * I_moon(moon_alpha) * 10**(-0.4 * k * X(Zen_moon)) * (1 - 10**(-0.4 * k * X(Zen)))
    else:
        return 0




"""
allskymaps python package.

Draws Allsky Nightsky Images with stars from the GAIA catalog

    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement.

and draws moonlight corresponding to the Model from Krisciunas et al.

   author = {{Krisciunas}, K. and {Schaefer}, B.~E.},
    title = "{A model of the brightness of moonlight}",
  journal = {\pasp},
     year = 1991,
    month = sep,
   volume = 103,
    pages = {1033-1039},
      doi = {10.1086/132921},
   adsurl = {http://adsabs.harvard.edu/abs/1991PASP..103.1033K},


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""

import numpy as np
import astropy.io.fits as pyfits
import ephem
import math
from scipy.ndimage import gaussian_filter, median_filter
from tqdm import tqdm

from .nsbtools import greatCircle, B_moon, B_sky, MagnLb, nLbMag



class FitsImage():
    def __init__(self, size, fov, alt, az, source, lon, lat, elevation, date_and_time, B_zero, k, use_mag):
        # create an empty FITS HDU with size x size for storing the model values
        self.imagesize = size
        a = np.empty((self.imagesize, self.imagesize))
        a.fill(np.nan)
        self.fitsfile = pyfits.PrimaryHDU(a)
        # create an PyEphem Observer and set the pressure to 0 -> No atmospheric corrections near the horizon
        self.site = ephem.Observer()
        self.site.pressure = 0
        self.site.elevation = elevation
        self.site.lon, self.site.lat = math.radians(lon), math.radians(lat)
        self.site.date = date_and_time

        self.moon = ephem.Moon()
        self.sun = ephem.Sun()
        self.moon.compute(self.site)
        self.sun.compute(self.site)

        self.zen_moon = np.pi / 2 - self.moon.alt
        self.az_moon = self.moon.az
        self.moon_alpha = math.acos(self.moon.moon_phase * 2 - 1)

        # all the other stuff
        self.resolution = math.radians(fov) / size
        self.fov = fov / 2.
        self.B_zero = B_zero
        self.k = k
        self.use_mag = use_mag

        if not source == None:
            pointing = source.name
            if source.name.lower().strip() == 'moon':
                self.alt = self.moon.alt
                self.az = self.moon.az
            else:
                self.source = ephem.FixedBody()
                self.source._ra, self.source._dec = source.ra, source.dec
                self.source.name = source.name
                self.source.compute(self.site)
                self.alt = self.source.alt
                self.az = self.source.az
                print('* %s at \tzen %.2f\taz %.2f' % (self.source.name, math.degrees(np.pi/2 - self.alt), self.az))
        else:
            pointing = 'n.a.'
            self.alt = math.radians(alt)
            self.az = math.radians(az)

        self.fitsfile.header['sunalt'] = (math.degrees(self.sun.alt), 'Degrees')
        self.fitsfile.header['moonalt'] = (math.degrees(self.moon.alt), 'Degrees')
        self.fitsfile.header['moonaz'] = (math.degrees(self.moon.az), 'Degrees')
        self.fitsfile.header['moonph'] = (self.moon.moon_phase, 'Moon Phase as Fraction of illuminated Surface')
        self.fitsfile.header['alt'] = (math.degrees(self.alt), 'Degrees (Pointing)')
        self.fitsfile.header['az'] = (math.degrees(self.az), 'Degrees (Pointing)')
        self.fitsfile.header['source'] = (pointing, 'Name of pointed source (if available)')
        self.fitsfile.header['fov'] = (2 * self.fov, 'Field of View shown in the Image')
        self.fitsfile.header['extinc'] = (self.k, 'Extincion Coefficient')
        self.fitsfile.header['bzero'] = (self.B_zero, 'Zenith brightness without moon')
        self.fitsfile.header['lat'] = (str(self.site.lat), 'Observer GPS Position')
        self.fitsfile.header['lon'] = (str(self.site.lon), 'Observer GPS Position')
        self.fitsfile.header['elev'] = (self.site.elevation, 'Observer Elevation [m above NN]')
        if self.use_mag:
            self.units = 'mag/arcsec^2'
        else:
            self.units = 'nLb'
        self.fitsfile.header['units'] = (self.units)


    def drawModel(self, gaiamap):
        sunalt = float(self.sun.alt)
        if sunalt > 0:
            sunset = self.site.next_setting(self.sun)
            # print fancy red error message
            message = 5 * '#' + ' Sun still up! Next sunset at %s ' % sunset + 5 * '#'
            print('\x1b[0;30;41m' + len(message) * '#')
            print(message)
            print(len(message) * '#' + '\x1b[0m')
            a = np.empty((self.imagesize, self.imagesize))
            a.fill(-1)
            self.fitsfile = pyfits.PrimaryHDU(a)
        elif sunalt < 0 and sunalt > math.radians(-18.0):
            # print fancy red error message
            message = 5 * '#' + ' Sunlight might have influence! sun alt= %.2f deg ' % math.degrees(sunalt) + 5 * '#'
            print('\x1b[0;30;41m' + len(message) * '#')
            print(message)
            print(len(message) * '#' + '\x1b[0m')
            a = np.zeros((self.imagesize, self.imagesize))
            self.fitsfile = pyfits.PrimaryHDU(a)
        else:
            offset_zen = float(np.pi/2 - self.alt)
            offset_az = float(self.az)
            offset_x = 0
            offset_y = -offset_zen * (1  /self.resolution)

            self.gaiamap = gaiamap

            print('Drawing Model...\t\t(reduce imagesize if slow)')
            # Model drawing starts here:
            self.fitsfile.header['ismodel'] = ('True', 'either Model or measurement')
            for x in tqdm(range(self.imagesize)):
                for y in range(self.imagesize):
                    az = (math.atan2((x - self.imagesize / 2. - offset_x),
                                     (y - self.imagesize / 2. - offset_y)) + offset_az)
                    zen = math.hypot((x - self.imagesize / 2. - offset_x),
                                     (y - self.imagesize / 2. - offset_y)) * self.resolution
                    moondist = greatCircle(zen, az, self.zen_moon, self.az_moon)
                    ra, dec = self.site.radec_of(az, math.pi / 2 - zen)
                    healpixid = self.gaiamap.healpix(math.degrees(ra), math.degrees(dec))
                    if (zen > math.radians(90.0)):
                        continue
                    else:
                        if self.use_mag:
                            if moondist > math.radians(5.0):
                                self.fitsfile.data[y, x] = nLbMag(
                                    B_moon(zen, az, self.zen_moon, self.az_moon, self.k, self.moon_alpha) +
                                    B_sky(zen, az, self.B_zero, self.k) +
                                    MagnLb(self.gaiamap.getBrightness(healpixid)))
                        else:
                            if moondist > math.radians(5.0):
                                self.fitsfile.data[y, x] = (
                                B_moon(zen, az, self.zen_moon, self.az_moon, self.k, self.moon_alpha) +
                                B_sky(zen, az, self.B_zero, self.k) +
                                MagnLb(self.gaiamap.getBrightness(healpixid)))

    def smooth_gauss(self, sigma):
        self.fitsfile.data = gaussian_filter(self.fitsfile.data, sigma=sigma)

    def median_filter(self, size):
        self.fitsfile.data = median_filter(self.fitsfile.data, size=int(size))

    def getStatistics(self):
        median, mean = np.nanmedian(self.fitsfile.data), np.nanmean(self.fitsfile.data)
        if not np.isnan(median) and not np.isnan(mean):
            self.fitsfile.header['median'] = (median, 'Median Image Brightness')
            self.fitsfile.header['mean'] = (mean, 'Mean Image Brightness')
        return median, mean
#
#



"""
Source catalog wrapper to quickly look up Ra Dec coordinates of sources.


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""

import os.path

class gammasource():
    """
    A source from my Catalog
    Constructor needs list with informations (splitted line from cat.csv)
    [name, ra, dec, type]
    """
    def __init__(self, info):
        self.name = info[0]
        self.ra = info[1]
        self.dec = info[2]
        self.type = info[3]

    def __repr__(self):
        return "<%s | %s | %s | %s>" % (self.name, self.ra, self.dec, self.type)

    def __str__(self):
        return "%s\ttype = %s\n(Ra %s | Dec %s)" % (self.name, self.type, self.ra, self.dec)

class mypycat():
    """
    a small catalog to store Ra/Dec coordinates of often used sources
    """
    def __init__(self):
        self.cat = self.loadCat()

    def loadCat(self):
        """
        load a csv file with catalog values.
        Syntax i.e.:
        Crab Pulsar;05 34 31.9;+22 00 52.2;PSR
        """
        try:
            home = os.path.join(os.path.expanduser("~"), ".nsb/")
            filepath = os.path.join(home, 'mypycat.txt')
            if not os.path.isfile(filepath):
                self.createMiniCat(filepath)
            catfile = open(filepath, 'r')
            cat = {}
            for line in catfile.readlines():
                if not line.startswith('#'):
                    cat[line.split(';')[0]] = [x for x in line.strip().split(';')]
            return cat

        except IOError as e:
            raise e


    def get(self, source):
        """
        Get a source from the catalog
        input: Source name - will try to find a match in the catalog, user will be asked if multiple are found
        returns: a tevcat.gammasource Object
        """
        source = source.strip()
        candidates = [value for key, value in self.cat.items() if source.lower() in key.lower()]
        if len(candidates) == 1:
            return gammasource(candidates[0])
        elif len(candidates) == 0:
            print('Source not found in catalog!')
            raise KeyError
        else:
            print('found %i sources matching %s:' % (len(candidates), source))
            for i in range(len(candidates)):
                print(str(i) + ') ' + candidates[i][0])
            i = eval(input('which one do you mean?\n'))
            return gammasource(candidates[i])


    def getAll(self):
        """
        Get All sources from the catalog
        returns: a list of cat.gammasource Objects
        """
        sourcelist = []
        for k, v in self.cat.items():
            sourcelist.append(gammasource(v))
        return sourcelist

    #cat = loadCat()

    def createMiniCat(self, filepath):
        print("seems you dont have a source catalog %s" % filepath)
        print("We will create a very minimalistic one. Feel free to add your own sources later (by hand)")
        with open(filepath, 'w') as f:
            f.write("# Name; Ra; Dec; comment\n"
                    "Crab Pulsar;05 34 31.9;+22 00 52.2;PSR\n"
                    "# Moon obviously has no Ra/Dec - this is just a dummy\n"
                    "Moon;00 00 0;00 00 0;none")
        print('SUCCESS! %s was saved to disc\nnow proceeding...\n' % filepath)



__author__ = "Matthias Buechele"
__author_email__ = "matthias.buechele@fau.de"
__maintainer__ = __author__
__email__ = __author_email__
__copyright__ = "Copyright 2017, Matthias Buechele"
__credits__ = ["Matthias Buechele"]
__license__ = "GNU GPL v3"
__shortname__ = "nsb"
__description__ = 'Draws nightsky allskymaps',
__long_description__ = ("Draws nightsky allskymaps corresponding to KRISCIUNAS Model of the Brightness of Moonlight,\n"
                "together with star data obtained from the GAIA public data release catalog.\n"
                "The result is a 2D Pixel array with physical brightness values for each sky position.")
__status__ = "Prototype"  # "Prototype", "Development", or "Production"
__version__ = "0.2.0"



"""
creates a skymap in healpix representation with data from GAIA.


    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement..


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""


import os.path
import healpy as hp
from ..gaia import create
from tqdm import tqdm


class Gaia(object):
    """docstring for gaia"""
    def __init__(self, level):
        self.level = level
        try:
            catalogname = 'healpixmap_level' + str(self.level) + '.txt'
            home = os.path.join(os.path.expanduser("~"), ".nsb/")
            filepath = os.path.join(home, catalogname)
            self.loadMap(filepath)
        except Exception as e:
            print('GAIA: ' + str(e))
            print('GAIA: Seems there is no gaia healpixmap with level %i.\n'
                  'GAIA: We will create a new one and try again.\n'
                  'GAIA: This might take a while...' % level)
            create.createMap(self.level)
            self.loadMap(filepath)
            # raise SystemExit

    def loadMap(self, filepath):
        print('GAIA: Opening gaia catalog map...\t(reduce heaplix level if slow)')
        f = open(filepath)
        self.map = []
        for line in tqdm(f.readlines(), mininterval=0.5):
            self.map.append([float(x) for x in line.split(',')])
        f.close()

    def healpix(self, ra, dec):
        return hp.ang2pix(2**self.level, ra, dec, nest=True, lonlat=True)

    def getBrightness(self, healpixid):
        return self.map[healpixid][4]

    def getFlux(self, healpixid):
        return self.map[healpixid][3]

    def getRa(self, healpixid):
        return self.map[healpixid][1]

    def getDec(self, healpixid):
        return self.map[healpixid][2]

    def getArcsec2(self, healpixid):
        return self.map[healpixid][5]

    def getNumber(self, healpixid):
        return self.map[healpixid][6]



from setuptools import setup
import nsb.__about__ as about
setup(
    name = 'nsb',
    packages = ['nsb', 'nsb/gaia', 'nsb/mypycat'],
    scripts = ['run_nsb'],
    version = about.__version__,
    description = about.__description__,
    long_description = about.__long_description__,
    author= about.__author__,
    author_email= about.__author_email__,
    url='https://pypi.python.org/pypi/nsb',
    install_requires=['healpy','tqdm', 'ephem', 'astropy', 'scipy', 'numpy', 'matplotlib'],
    package_dir={'nsb': 'nsb'},
    package_data={'nsb': ['gaia/ducaticat.txt']},
    entry_points = {
        "console_scripts": ['nsb = nsb.nsb:main']
        },
    classifiers=[
              'Development Status :: 3 - Alpha',
              'Environment :: Console',
              'Intended Audience :: Science/Research',
              'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
              'Operating System :: POSIX :: Linux',
              'Topic :: Scientific/Engineering :: Astronomy',
              'Topic :: Scientific/Engineering :: Atmospheric Science',
              'Topic :: Scientific/Engineering :: Physics',
              ],
)
# package_data={'nsb': ['gaia/ducaticat.txt','mypycat/cat.csv', 'config.cfg']},


"""
allskymaps python package.

Draws Allsky Nightsky Images with stars from the GAIA catalog

    This work has made use of data from the European Space Agency (ESA)
    mission {\\it Gaia} (\\url{https://www.cosmos.esa.int/gaia}), processed by
    the {\\it Gaia} Data Processing and Analysis Consortium (DPAC,
    \\url{https://www.cosmos.esa.int/web/gaia/dpac/consortium}). Funding
    for the DPAC has been provided by national institutions, in particular
    the institutions participating in the {\\it Gaia} Multilateral Agreement.

and draws moonlight corresponding to the Model from Krisciunas et al.

   author = {{Krisciunas}, K. and {Schaefer}, B.~E.},
    title = "{A model of the brightness of moonlight}",
  journal = {\pasp},
     year = 1991,
    month = sep,
   volume = 103,
    pages = {1033-1039},
      doi = {10.1086/132921},
   adsurl = {http://adsabs.harvard.edu/abs/1991PASP..103.1033K},


This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>

Created and maintained by Matthias Buechele [FAU].

"""

import os

from .nsbtools import makeDate



class TheConfiguration():
    def __init__(self):
        pass

    def readStandardConfig(self):
        try:
            home = os.path.join(os.path.expanduser("~"), ".nsb/")
            self.config_filepath = os.path.join(home,'config.cfg')
            self.readConfig(self.config_filepath)
        except Exception as e:
            print('%s not found! \nWill now try to create a default one!' % self.config_filepath)
            print(e)
            self.createConfig(self.config_filepath)
            self.readConfig(self.config_filepath)

    def readConfig(self, configfile):
        self.file = open(configfile, 'r')
        self.config = {}
        try:
            for line in self.file:
                line = line.strip()
                if not line.startswith('#'):
                    self.config[line.split('=')[0].strip()] = line.split('=')[1].strip()
            t1, t2 = self.config['time'].split(' ')[0], self.config['time'].split(' ')[1]
            self.config['time'] = makeDate(t1, t2)
            print('successfully opened %s' % configfile)
        except Exception as e:
            print('\nSomething went wrong while reading %s' % configfile)
            print(e)
            pass
        self.file.close()

    def createConfig(self, configfile):
        with open(configfile, 'w+') as f:
            f.write("# allskymaps Config File\n"
                    "#\n"
                    "# Date and Time\n"
                    "time = today now\n"
                    "#time = 2017/01/18 23:30:00\n"
                    "#\n"
                    "# Observer Location (HESS is at 16.5028 -23.27280 @ 1800.0)\n"
                    "Lon = 16.5028\n"
                    "Lat = -23.27280\n"
                    "elevation = 1800.\n"
                    "#\n"
                    "# dark zenith night sky brightness (units nanoLambert)\n"
                    "B = 77.0\n"
                    "#\n"
                    "# Extinction coefficient (units mags per air mass)\n"
                    "k = 0.479\n"
                    "#\n"
                    "# define output units mag/arcsec^2 (instead of nanoLamberts)\n"
                    "use_mag = False\n"
                    "#\n"
                    "# output Imagesize in Pixels (its gonna be square)\n"
                    "image_size = 200\n"
                    "#\n"
                    "# field ov view of the sky images.\n"
                    "fov = 50\n"
                    "#\n"
                    "# observation position in the sky [deg]\n"
                    "alt = 90.0\n"
                    "az  = 0.0\n"
                    "#\n"
                    "# Level of HealPixMap used for plotting gaia catalog\n"
                    "healpixlevel = 7\n"
                    "#\n"
                    "# Gaussian smoothing kernel [pixel]\n"
                    "gauss = 0.0\n"
                    "#\n")
        print('SUCCESS! %s with default values was saved to disc\nnow proceeding...\n' % configfile)
